@@ -276,6 +276,9 @@ thunar_folder_finalize (GObject *object)
 {
   ThunarFolder *folder = THUNAR_FOLDER (object);
 
+  if (folder->corresponding_file)
+    thunar_file_unwatch (folder->corresponding_file);
+
   /* disconnect from the ThunarFileMonitor instance */
   g_signal_handlers_disconnect_matched (folder->file_monitor, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, folder);
   g_object_unref (folder->file_monitor);
@@ -356,7 +359,11 @@ thunar_folder_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_CORRESPONDING_FILE:
+      if (folder->corresponding_file)
+        thunar_file_unwatch (folder->corresponding_file);
       folder->corresponding_file = g_value_dup_object (value);
+      if (folder->corresponding_file)
+        thunar_file_watch (folder->corresponding_file);
       break;
 
     case PROP_LOADING:
@@ -414,6 +421,8 @@ thunar_folder_files_ready (ThunarJob    *job,
 static gboolean
 thunar_folder_content_type_loader_idle (gpointer data)
 {
+  _thunar_return_val_if_fail (THUNAR_IS_FOLDER (data), FALSE);
+
   ThunarFolder *folder = THUNAR_FOLDER (data);
   GList        *lp;
 
@@ -440,6 +449,8 @@ thunar_folder_content_type_loader_idle (gpointer data)
 static void
 thunar_folder_content_type_loader_idle_destroyed (gpointer data)
 {
+  _thunar_return_if_fail (THUNAR_IS_FOLDER (data));
+
   THUNAR_FOLDER (data)->content_type_idle_id = 0;
 }
 
@@ -560,9 +571,12 @@ thunar_folder_finished (ExoJob       *job,
     }
 
   /* we did it, the folder is loaded */
-  g_signal_handlers_disconnect_matched (folder->job, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, folder);
-  g_object_unref (folder->job);
-  folder->job = NULL;
+  if (G_LIKELY (folder->job != NULL))
+    {
+      g_signal_handlers_disconnect_matched (folder->job, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, folder);
+      g_object_unref (folder->job);
+      folder->job = NULL;
+    }
 
   /* restart the content type idle loader */
   thunar_folder_content_type_loader (folder);
@@ -773,27 +787,30 @@ thunar_folder_monitor (GFileMonitor     *monitor,
             {
               /* destroy the old file and update the new one */
               thunar_file_destroy (lp->data);
-              file = thunar_file_get(other_file, NULL);
-              if (file != NULL && THUNAR_IS_FILE (file))
+              if (other_file != NULL)
                 {
-                  thunar_file_reload (file);
-
-                  /* if source and target folders are different, also tell
-                     the target folder to reload for the changes */
-                  if (thunar_file_has_parent (file))
+                  file = thunar_file_get(other_file, NULL);
+                  if (file != NULL && THUNAR_IS_FILE (file))
                     {
-                      other_parent = thunar_file_get_parent (file, NULL);
-                      if (other_parent &&
-                          !g_file_equal (thunar_file_get_file(folder->corresponding_file),
-                                         thunar_file_get_file(other_parent)))
-                        {
-                          thunar_file_reload (other_parent);
-                          g_object_unref (other_parent);
-                        }
-                    }
+                      thunar_file_reload (file);
 
-                  /* drop reference on the other file */
-                  g_object_unref (file);
+                      /* if source and target folders are different, also tell
+                         the target folder to reload for the changes */
+                      if (thunar_file_has_parent (file))
+                        {
+                          other_parent = thunar_file_get_parent (file, NULL);
+                          if (other_parent &&
+                              !g_file_equal (thunar_file_get_file(folder->corresponding_file),
+                                             thunar_file_get_file(other_parent)))
+                            {
+                              thunar_file_reload (other_parent);
+                              g_object_unref (other_parent);
+                            }
+                        }
+
+                      /* drop reference on the other file */
+                      g_object_unref (file);
+                    }
                 }
 
               /* reload the folder of the source file */
